#include <cmath>

#include "canvas/BackGround.hpp"
#include "canvas/BackGroundColors.hpp"
#include "Math.hpp"
#include "values/mrand.hpp"

#include <raylib.h>
#include <rlgl.h>
#include <raymath.h>

// Public
void BackGround::init(int canvasSize)
{
  this->canvasSize = canvasSize;
  sun.init();
}

void BackGround::deinit()
{
  sun.deinit();
}

void BackGround::draw(Dna *dna)
{
  m_dna = dna;
  mountenSeed = dna->mountenSeed;
  starSeed = dna->starSeed;

  BackGroundColors::setColor(m_dna->colorSet, m_dna->time);

  if (m_dna->colorSet == 3)
  {
    ClearBackground(BackGroundColors::backGroundColor);
    drawStars();
  }
  else
  {
    DrawRectangleGradientV(0, 0, canvasSize, canvasSize, ColorAdd(BackGroundColors::backGroundColor, 60), BackGroundColors::backGroundColor);
  }

  drawSun();
  drawMounten(20, (int)(mounten1min * canvasSize), (int)(mounten1max * canvasSize), BackGroundColors::MountenColor1);
  drawMounten(21, (int)(mounten2min * canvasSize), (int)(mounten2max * canvasSize), BackGroundColors::MountenColor2);
  drawMounten(23, (int)(mounten3min * canvasSize), (int)(mounten3max * canvasSize), BackGroundColors::MountenColor3);
}

void BackGround::drawStars()
{
  rlSetTexture(1);
  rlBegin(RL_TRIANGLES);
  rlNormal3f(0.0f, 0.0f, 1.0f);
  bool star = true;

  for (size_t i = 0; i < numOfStarts; i++)
  {
    int x = mrand::getFloat(&starSeed) * canvasSize;
    int y = mrand::getFloat(&starSeed) * canvasSize;
    int weight = mrand::getFloat(&starSeed) * 3 + 1;
    float alph = Normalize(mrand::getFloat(&starSeed), 0.04f, 0.8f);
    Color color = ColorLerp(BackGroundColors::backGroundColor, BackGroundColors::starColor, alph);

    rlColor4ub(color.r, color.g, color.b, color.a);
    if (star)
    {
      // topLeft
      rlVertex2f(x, y);
      // bottomLeft
      rlVertex2f(x, y + weight);
      // topRight
      rlVertex2f(x + weight, y);
    }
    else
    {
      // bottomRight
      rlVertex2f(x + weight, y + weight);
      // topRight
      rlVertex2f(x + weight, y);
      // bottomLeft
      rlVertex2f(x, y + weight);
    }
    star = !star;
  }
  rlEnd();
  rlSetTexture(0);
}

void BackGround::drawSun()
{
  int r = ((m_dna->moon.y * (maxSizeOfMoon - minSizeOfMoon)) + minSizeOfMoon) * canvasSize;
  int xpos = Lerp(canvasSize * moonXOffset, canvasSize - canvasSize * moonXOffset, m_dna->moon.x);
  int ypos = Lerp(canvasSize * moonXOffset, maxYPosOfMoon * canvasSize, m_dna->moon.y);

  if (m_dna->colorSet == 3)
  {
    sun.setColor(BackGroundColors::moonColor);
    r = ((m_dna->moon.size * (maxSizeOfMoon - minSizeOfMoon)) + minSizeOfMoon) * canvasSize;
  }
  else
  {
    Color color = {0};
    color.r = 255;
    color.g = std::lerp(200, 50, m_dna->moon.y);
    color.b = std::lerp(50, 0, m_dna->moon.y);
    color.a = 255;

    sun.setColor(color);
  }
  sun.draw(xpos, ypos, r);
}

void BackGround::drawMounten(size_t mountenSegments, int min, int max, Color color)
{
  int x = 0;
  int diff = canvasSize / (mountenSegments - 1);

  float p = mrand::getFloat(&mountenSeed);

  int y = Lerp(min, max, p);
  rlSetTexture(1);
  rlBegin(RL_TRIANGLES);
  rlNormal3f(0.0f, 0.0f, 1.0f);
  rlColor4ub(color.r, color.g, color.b, color.a);

  for (size_t i = 1; i <= mountenSegments; i++)
  {
    // topLeft
    rlVertex2f(x, y);
    // bottomLeft
    rlVertex2f(x, canvasSize);

    p = mrand::getFloat(&mountenSeed);
    y = Lerp(min, max, p);
    x += diff;
    // topRight
    rlVertex2f(x, y);

    // bottomRight
    rlVertex2f(x, canvasSize);
    // topRight
    rlVertex2f(x, y);

    rlVertex2f(x - diff, canvasSize);
  }
  rlEnd();
  rlSetTexture(0);
}